<?php
namespace Teknisa\MYO\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class CentroDeCusto extends \Teknisa\GER\Controllers\TeknisaCrud{
    
    protected $crudService;
    protected $parameterBag;
    protected $util;
    protected $dataSourceManager;
    protected $newCode;
    protected $environment;

	public function __construct($crudService, $parameterBag, $util, $dataSourceManager, $environment, $newCode) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->util = $util;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCode = $newCode;
        $this->environment = $environment;
    }
    
    public function save(DTO\Request $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "ccuscliefilial";
            $row = $request->getDataSet()->getRows()[0];
            
            if ($row["__is_new"]) {
                $this->util->validateDuplicatedRecords($row, $this->dataSourceName, array("CDFILIAL", "CDCLIENTE", "CDCCUSCLIEFILIAL"));
            }

            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function export(DTO\Request $request, DTO\Response $response){
        try {
            $this->dataSourceName = "ccuscliefilial";
            $data =  $request->getDataSet()->getRows();
            if(!empty($data)){
                $centroCusto = $data[0];
                $row = $data[1];
                $dataSave = array();
                $dataClienfilial = $this->getClienFilial($centroCusto, $row);
                if(!empty($dataClienfilial)){
                    foreach($dataClienfilial as $clienfilial){
                        if($clienfilial['CDCLIENTE'] !== $row['CDCLIENTE'] || $clienfilial['CDFILIAL'] !== $row['CDFILIAL']){
                            foreach($centroCusto as $ccusto){
                                if(!$this->existsCentroCusto($ccusto, $clienfilial)){
                                    $rowSave = $this->buildRowToSave($row, $clienfilial, $ccusto);
                                    array_push($dataSave, $rowSave);
                                }
                            }
                        }
                    }
                }
                $dataSet = new DataSet($this->dataSourceName, $dataSave);
                $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                $rowsToSave = $this->crudService->getRowsToSave($newRequest, $this->dataSourceName);
                $this->crudService->save($this->dataSourceName, $rowsToSave);
            }
        } catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    private function buildRowToSave($row, $clienfilial, $ccusto){
        $nrorg = $this->environment->getNrOrgTrab();
        $currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
        return array(
            "__is_new" => true, 
            "changed"  => true,
            "NRCCUSCLIEFILIAL"  => $this->newCode->getCodeByTableName($row['paramsNewCode']['__new_code_pk'], $nrorg),
            "CDFILIAL"  => $clienfilial['CDFILIAL'],
            "CDCLIENTE" => $clienfilial['CDCLIENTE'],
            "CDCCUSCLIEFILIAL" => $ccusto['CDCCUSCLIEFILIAL'],
            "NMCCUSCLIEFILIAL" => $ccusto['NMCCUSCLIEFILIAL'],
            "NRORG"    => $this->environment->getNrorgTrab(),
            "DTULTATU" => $currentTime,
            "NRORGULTATU" => $this->environment->getNrOrgTrab(),
            "CDOPERULTATU" => $this->environment->getCdOperador(),
            "DTINCLUSAO" => $currentTime,
            "NRORGINCLUSAO" => $this->environment->getNrOrgTrab(),
            "CDOPERINCLUSAO" => $this->environment->getCdOperador()
        );
    }

    private function existsCentroCusto($ccusto, $clienfilial) {
        $conditions = array(
            array(
                "columnName" => "NRORG",
                "operator"   => '=',
                "value"      =>  $this->environment->getNrorgTrab()
            ),
            array(
                "columnName" => "CDFILIAL",
                "operator"   => '=',
                "value"      => $clienfilial["CDFILIAL"]
            ),
            array(
                "columnName" => "CDCLIENTE",
                "operator"   => '=',
                "value"      => $clienfilial["CDCLIENTE"]
            ),
            array(
                "columnName" => "CDCCUSCLIEFILIAL",
                "operator"   => '=',
                "value"      => $ccusto["CDCCUSCLIEFILIAL"]
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet("ccuscliefilial", $conditions);
        $rows = $dataSet->getRows();
        return !empty($rows);
    }

    private function getClienFilial($centroCusto, $row) {
        $conditions = array(
            array(
                "columnName" => "NRORG",
                "operator"   => '=',
                "value"      =>  $this->environment->getNrorgTrab()
            ),
            array(
                "columnName" => "NMFILIAL",
                "operator"   => '=',
                "value"      => $row['NMFILIAL']
            ),
            array(
                "columnName" => "CLIENTE",
                "operator"   => '=',
                "value"      => $row['CLIENTE']
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet("clienfilial_ccuscliefilial", $conditions);
        $rows = $dataSet->getRows();
        return $rows;
    }
}