<?php
namespace Teknisa\MYO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ControlePedidosReport extends \Zeedhi\Framework\Controller\Report {

    protected $newCode;
    protected $birtStrategy;
    protected $environment;
    protected $crudService;
    protected $dataSourceManager;
    protected $dataSourceName;
    protected $nrlista;

    const TABLE_LISTA = "LISTA";

    public function __construct(ReportService $reportService, $newCode, $birtStrategy, $environment, $crudService, $dataSourceManager) {
        $this->reportService = $reportService;
        $this->newCode       = $newCode;
        $this->birtStrategy = $birtStrategy;
        $this->environment   = $environment;
        $this->crudService   = $crudService;
        $this->dataSourceManager = $dataSourceManager;
    }

    protected $reportMapping = array (
        'MYO00200' =>
        array (
            'strategy' => 'BIRT', 
            'parameterMapping' =>
            array (
                'DTINICIAL'         => 'DTINICIAL',
                'DTFINAL'           => 'DTFINAL',
                'NRORG'             => 'NRORG',
                'NRLISTA'           => 'NRLISTA',
                'CLIENTES'          => 'CLIENTES',
                'CENTROCUSTO'       => 'CENTROCUSTO',
                'STATUS'            => 'STATUS'
            )
        )
    );

    /**
     * prepareParameters
     * Verifica e realiza se necessário fazer o implode do parametro
     * caso não, retorna 'T' 
     *
     * @param Row $row
     * @param string $name
     *
     * @return string
     */
    private function prepareParameters($row, $name){
        if(isset($row[$name])) {
            if (!empty($row[$name])) {
                return '_'.implode('_', $row[$name]).'_';
            }
        }
        return 'T';
    }

    /**
     * getValueByField
     *
     * @param Row $row
     * @param string $checkField
     * @param string $field
     *
     * @return string
     */
    private function getValueByField($row, $checkField, $field){
        if(isset($row[$checkField])) {
            return 'T';
        }else {
            return $row[$field];
        }
    }

    /**
     * getCdfiliserv
     * 
     * @param Row $row
     *
     * @return Array
     */
    private function getCdfiliserv($row) {
        if ($row["TSERVICO"] !== "S") {
            return $row["CHAVE"];
        } else {
            if(isset($row['CDGRUPFILI']) && empty($row['CDGRUPFILI'])){
                 $row['CDGRUPFILI'] = "T";
            }
            $conditions = array(
                array(
                    "columnName" => "CDGRUPFILI",
                    "operator"   => '=',
                    "value"      => $row['CDGRUPFILI']
                ),
                array(
                    "columnName" => "CDFILIAL",
                    "operator"   => '=',
                    "value"      => $this->getValueByField($row, "TFILIAL", "CDFILIAL")
                ),
                array(
                    "columnName" => "CDGRUPSERV",
                    "operator"   => '=',
                    "value"      => $this->getValueByField($row, "TGRUPSERV", "CDGRUPSERV")
                )
            );
            
            $dataSet = $this->crudService->getFilteredDataSet("servico_filial_grupserv_filgrupfili", $conditions);
            $rows = $dataSet->getRows();
            $cdfiliserv = array();
            foreach ($rows as $row) {
                array_push($cdfiliserv, $row["CHAVE"]);
            }
            return $cdfiliserv;
        }
    }

    /**
     * insertLista
     * Executa o insert da lista
     *
     * @param Array $cdlistaArray
     *
     * @return int|string
     */
    private function insertLista($cdlistaArray) {
        $this->dataSourceName = strtolower(self::TABLE_LISTA);
        $nrlista = $this->newCode->getCodeByTableName(self::TABLE_LISTA, $this->environment->getNrOrg());
        $rows = array();
        foreach ($cdlistaArray as $cdlista) {
            $row = array(
                "NRLISTA"  => $nrlista,
                "CDLISTA"  => $cdlista,
                "__is_new" => true,
                "NRORG"    => $this->environment->getNrorgTrab()
            );
            array_push($rows, $row);
        }
        $dataSet = new DataSet($this->dataSourceName, $rows);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rowsToSave = $this->crudService->getRowsToSave($request, $this->dataSourceName);
        $this->crudService->save($this->dataSourceName, $rowsToSave);
        return $nrlista;
    }

    /**
     * beforeProcessReport
     * Chamado pelo Report::createRemoteReport, cria as listas e adiciona os
     * campos na $row conforme especificado pelo relatório.
     *
     * @param Row $request
     * @param Response $response
     * 
     * @return void
     */
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $cdfiliserv = $this->getCdfiliserv($row);
        
        $this->nrlista = $this->insertLista($cdfiliserv);

        $row["NRLISTA"] = $this->nrlista;
        $row["NRORG"] = $this->environment->getNrOrg();
        $row["CLIENTES"] = $this->prepareParameters($row, "CLIENT");
        $row["CENTROCUSTO"] = $this->prepareParameters($row, "CENTROCUSTO");
        $row["STATUS"] = $this->prepareParameters($row, "STATUS");
        
        $this->birtStrategy->setReportFormat($row['QR2TYPE']);
    }

    /**
     * deleteLista
     * Executa o delete da lista
     *
     * @param int|string $nrLista
     *
     * @return Void
     */
    private function deleteLista($nrlista) {
        $dataSourceName = strtolower(self::TABLE_LISTA);
        
        $conditions = array(
            array(
                "columnName" => "NRLISTA",
                "operator"   => '=',
                "value"      => $nrlista
            )
        );
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
        $this->dataSourceManager->delete($dataSet);
    }

    /**
     * afterProcessReport
     * Chamado pelo Report::createRemoteReport, deleta as listas após serem
     * usadas para gerar o relatório
     *
     * @param Row $request
     * @param Response $response
     * @param $report Report URL
     *
     * @return void
     */
    public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report) {
        // $this->deleteLista($this->nrlista);
    }

}